@@ -25,21 +25,23 @@ module Test.Hspec (
 , after
 , around
 , parallel
+, runIO
 
 -- * Running a spec
 , hspec
 ) where
 
+import           Control.Exception (finally)
+
 import           Test.Hspec.Core.Type hiding (describe, it)
 import           Test.Hspec.Runner
 import           Test.Hspec.HUnit ()
 import           Test.Hspec.Expectations
-import           Test.Hspec.Core (mapSpecItem)
 import qualified Test.Hspec.Core as Core
 
 -- | Combine a list of specs into a larger spec.
 describe :: String -> Spec -> Spec
-describe label action = fromSpecList [Core.describe label (runSpecM action)]
+describe label action = fromSpecList [Core.describe label [BuildSpecs $ runSpecM action]]
 
 -- | An alias for `describe`.
 context :: String -> Spec -> Spec
@@ -84,7 +86,7 @@ before action = around (action >>)
 
 -- | Run a custom action after every spec item.
 after :: IO () -> Spec -> Spec
-after action = around (>> action)
+after action = around (`finally` action)
 
 -- | Run a custom action before and/or after every spec item.
 around :: (IO () -> IO ()) -> Spec -> Spec
