@@ -3,15 +3,21 @@ module Test.Hspec.Compat (
   showType
 , showFullType
 , readMaybe
+, lookupEnv
 , module Data.IORef
 #if !MIN_VERSION_base(4,6,0)
 , modifyIORef'
 #endif
 ) where
 
+#if !MIN_VERSION_base(4,3,0)
+import           Control.Monad.Trans.Error () -- for Monad (Either e)
+#endif
+
 import           Data.Typeable (Typeable, typeOf, typeRepTyCon)
 import           Text.Read
 import           Data.IORef
+import           System.Environment
 
 #if MIN_VERSION_base(4,4,0)
 import           Data.Typeable.Internal (tyConModule, tyConName)
@@ -19,7 +25,6 @@ import           Data.Typeable.Internal (tyConModule, tyConName)
 
 #if !MIN_VERSION_base(4,6,0)
 import qualified Text.ParserCombinators.ReadP as P
-import Prelude
 #endif
 
 #if !MIN_VERSION_base(4,6,0)
@@ -51,6 +56,13 @@ readMaybe :: Read a => String -> Maybe a
 readMaybe s = case readEither s of
                 Left _  -> Nothing
                 Right a -> Just a
+
+-- | Return the value of the environment variable @var@, or @Nothing@ if
+-- there is no such value.
+--
+-- For POSIX users, this is equivalent to 'System.Posix.Env.getEnv'.
+lookupEnv :: String -> IO (Maybe String)
+lookupEnv k = lookup k `fmap` getEnvironment
 #endif
 
 showType :: Typeable a => a -> String
