@@ -23,40 +23,40 @@ import           Control.Applicative
 import           Data.Monoid
 import           Data.Maybe
 import           System.IO
-import           System.Environment
+import           System.Environment (getProgName, getArgs, withArgs)
 import           System.Exit
 import qualified Control.Exception as E
 
 import           System.Console.ANSI (hHideCursor, hShowCursor)
 import qualified Test.QuickCheck as QC
-import           System.Random (newStdGen)
 import           Control.Monad.IO.Class (liftIO)
 
-import           Test.Hspec.Util
+import           Test.Hspec.Compat (lookupEnv)
+import           Test.Hspec.Util (Path)
 import           Test.Hspec.Core.Type
 import           Test.Hspec.Config
 import           Test.Hspec.Formatters
 import           Test.Hspec.Formatters.Internal
 import           Test.Hspec.FailureReport
+import           Test.Hspec.Core.QuickCheckUtil
 
 import           Test.Hspec.Options (Options(..), ColorMode(..), defaultOptions)
+import           Test.Hspec.Runner.Tree
 import           Test.Hspec.Runner.Eval
 
 -- | Filter specs by given predicate.
 --
 -- The predicate takes a list of "describe" labels and a "requirement".
-filterSpecs :: (Path -> Bool) -> [SpecTree] -> [SpecTree]
+filterSpecs :: (Path -> Bool) -> [Tree a] -> [Tree a]
 filterSpecs p = goSpecs []
   where
-    goSpecs :: [String] -> [SpecTree] -> [SpecTree]
     goSpecs groups = mapMaybe (goSpec groups)
 
-    goSpec :: [String] -> SpecTree -> Maybe SpecTree
     goSpec groups spec = case spec of
-      SpecItem item -> guard (p (groups, itemRequirement item)) >> return spec
-      SpecGroup group specs     -> case goSpecs (groups ++ [group]) specs of
+      Leaf requirement _ -> guard (p (groups, requirement)) >> return spec
+      Node group specs -> case goSpecs (groups ++ [group]) specs of
         [] -> Nothing
-        xs -> Just (SpecGroup group xs)
+        xs -> Just (Node group xs)
 
 -- | Run given spec and write a report to `stdout`.
 -- Exit with `exitFailure` if at least one spec item fails.
@@ -70,16 +70,14 @@ hspecWithFormatter formatter spec = do
   f <- toFormatter formatter
   hspecWithOptions defaultOptions {optionsFormatter = f} spec
 
--- Add a StdGen to configQuickCheckArgs if there is none.  That way the same
--- seed is used for all properties.  This helps with --seed and --rerun.
-ensureStdGen :: Config -> IO Config
-ensureStdGen c = case QC.replay qcArgs of
+-- Add a seed to given config if there is none.  That way the same seed is used
+-- for all properties.  This helps with --seed and --rerun.
+ensureSeed :: Config -> IO Config
+ensureSeed c = case configQuickCheckSeed c of
   Nothing -> do
-    stdGen <- newStdGen
-    return c {configQuickCheckArgs = qcArgs {QC.replay = Just (stdGen, 0)}}
+    seed <- newSeed
+    return c {configQuickCheckSeed = Just (fromIntegral seed)}
   _       -> return c
-  where
-    qcArgs = configQuickCheckArgs c
 
 -- | Run given spec with custom options.
 -- This is similar to `hspec`, but more flexible.
@@ -106,16 +104,21 @@ hspecResult = hspecWith defaultConfig
 -- items.  If you need this, you have to check the `Summary` yourself and act
 -- accordingly.
 hspecWith :: Config -> Spec -> IO Summary
-hspecWith c_ spec = withHandle c_ $ \h -> do
-  c <- ensureStdGen c_
+hspecWith c_ spec_ = withHandle c_ $ \h -> do
+  c <- ensureSeed c_
   let formatter = configFormatter c
-      seed = (stdGenToInteger . fst . fromJust . QC.replay . configQuickCheckArgs) c
+      seed = (fromJust . configQuickCheckSeed) c
+      qcArgs = configQuickCheckArgs c
+      spec
+        | configDryRun c = mapSpecItem markSuccess spec_
+        | otherwise      = spec_
 
   useColor <- doesUseColor h c
+  filteredSpec <- maybe id filterSpecs (configFilterPredicate c) <$> toTree spec
 
   withHiddenCursor useColor h $
     runFormatM useColor (configHtmlOutput c) (configPrintCpuTime c) seed h $ do
-      runFormatter useColor h c formatter (maybe id filterSpecs (configFilterPredicate c) $ runSpecM spec) `finally_` do
+      runFormatter useColor h c formatter filteredSpec `finally_` do
         failedFormatter formatter
 
       footerFormatter formatter
@@ -124,9 +127,9 @@ hspecWith c_ spec = withHandle c_ $ \h -> do
       xs <- map failureRecordPath <$> getFailMessages
       liftIO $ writeFailureReport FailureReport {
           failureReportSeed = seed
-        , failureReportMaxSuccess = QC.maxSuccess (configQuickCheckArgs c)
-        , failureReportMaxSize = QC.maxSize (configQuickCheckArgs c)
-        , failureReportMaxDiscardRatio = QC.maxDiscardRatio (configQuickCheckArgs c)
+        , failureReportMaxSuccess = QC.maxSuccess qcArgs
+        , failureReportMaxSize = QC.maxSize qcArgs
+        , failureReportMaxDiscardRatio = QC.maxDiscardRatio qcArgs
         , failureReportPaths = xs
         }
 
@@ -139,7 +142,7 @@ hspecWith c_ spec = withHandle c_ $ \h -> do
 
     doesUseColor :: Handle -> Config -> IO Bool
     doesUseColor h c = case configColorMode c of
-      ColorAuto  -> hIsTerminalDevice h
+      ColorAuto  -> (&&) <$> hIsTerminalDevice h <*> (not <$> isDumb)
       ColorNever -> return False
       ColorAlways -> return True
 
@@ -148,6 +151,12 @@ hspecWith c_ spec = withHandle c_ $ \h -> do
       Left h -> action h
       Right path -> withFile path WriteMode action
 
+isDumb :: IO Bool
+isDumb = maybe False (== "dumb") <$> lookupEnv "TERM"
+
+markSuccess :: Item -> Item
+markSuccess item = item {itemExample = evaluateExample Success}
+
 -- | Summary of a test run.
 data Summary = Summary {
   summaryExamples :: Int
