@@ -2,6 +2,7 @@
 module Test.Hspec.CompatSpec (main, spec) where
 
 import           Helper
+import           System.SetEnv
 
 import           Test.Hspec.Compat
 import           Data.Typeable
@@ -21,3 +22,12 @@ spec = do
   describe "showFullType (currently unused)" $ do
     it "shows fully qualified name of type" $ do
       showFullType SomeType `shouldBe` "Test.Hspec.CompatSpec.SomeType"
+
+  describe "lookupEnv" $ do
+    it "returns value of specified environment variable" $ do
+      setEnv "FOO" "bar"
+      lookupEnv "FOO" `shouldReturn` Just "bar"
+
+    it "returns Nothing if specified environment variable is not set" $ do
+      unsetEnv "FOO"
+      lookupEnv "FOO" `shouldReturn` Nothing
