@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 module Test.Hspec.RunnerSpec (main, spec) where
 
 import           Helper
@@ -5,10 +6,11 @@ import           System.IO (stderr)
 import           Control.Monad
 import           System.Environment (withArgs, withProgName, getArgs)
 import           System.Exit
+import           Control.Concurrent
 import qualified Control.Exception as E
 import           Mock
 import           System.SetEnv
-import           Test.Hspec.Util (getEnv)
+import           Test.Hspec.Compat
 
 import           Test.Hspec.FailureReport (FailureReport(..))
 import qualified Test.Hspec as H
@@ -24,6 +26,22 @@ main = hspec spec
 quickCheckOptions :: [([Char], Args -> Int)]
 quickCheckOptions = [("--qc-max-success", QC.maxSuccess), ("--qc-max-size", QC.maxSize), ("--qc-max-discard", QC.maxDiscardRatio)]
 
+prop_foo :: Integer -> Bool
+prop_foo = (/= 26)
+
+runPropFoo :: [String] -> IO String
+runPropFoo args = fmap (unlines . normalizeSummary . lines) . capture_ . ignoreExitCode . withArgs args . H.hspec $ H.it "foo" $ property prop_foo
+
+prop_foo_result_with_seed_42 :: String
+prop_foo_result_with_seed_42 = unlines [
+#if MIN_VERSION_QuickCheck(2,7,0)
+            "Falsifiable (after 31 tests): "
+#else
+            "Falsifiable (after 30 tests): "
+#endif
+          , "26"
+          ]
+
 spec :: Spec
 spec = do
   describe "hspec" $ do
@@ -59,7 +77,7 @@ spec = do
             H.it "example 2" False
         H.describe "baz" $ do
           H.it "example 3" False
-      getEnv "HSPEC_FAILURES" `shouldReturn` (Just . show) FailureReport {
+      lookupEnv "HSPEC_FAILURES" `shouldReturn` (Just . show) FailureReport {
           failureReportSeed = 23
         , failureReportMaxSuccess = 100
         , failureReportMaxSize = 100
@@ -86,20 +104,8 @@ spec = do
         r1 `shouldSatisfy` elem "3 examples, 3 failures"
 
       it "reuses the same seed" $ do
-        let runSpec_ = (captureLines . ignoreExitCode . H.hspec) $ do
-              H.it "foo" $ property $ (/= (26 :: Integer))
-
-        r0 <- withArgs ["--seed", "2413421499272008081"] runSpec_
-        r0 `shouldContain` [
-            "Falsifiable (after 66 tests): "
-          , "26"
-          ]
-
-        r1 <- withArgs ["-r"] runSpec_
-        r1 `shouldContain` [
-            "Falsifiable (after 66 tests): "
-          , "26"
-          ]
+        r <- runPropFoo ["--seed", "42"]
+        runPropFoo ["-r"] `shouldReturn` r
 
       forM_ quickCheckOptions $ \(flag, accessor) -> do
         it ("reuses same " ++ flag) $ do
@@ -138,11 +144,19 @@ spec = do
 
     context "when interrupted with ctrl-c" $ do
       it "prints summary immediately" $ do
-        r <- captureLines . ignoreUserInterrupt . withArgs ["--seed", "23"] . H.hspec $ do
-          H.it "foo" False
-          H.it "bar" $ do
-            E.throwIO E.UserInterrupt :: IO ()
-          H.it "baz" True
+        mvar <- newEmptyMVar
+        sync <- newEmptyMVar
+        threadId <- forkIO $ do
+          r <- captureLines . ignoreUserInterrupt . withArgs ["--seed", "23"] . H.hspec $ do
+            H.it "foo" False
+            H.it "bar" $ do
+              putMVar sync ()
+              threadDelay 1000000
+            H.it "baz" True
+          putMVar mvar r
+        takeMVar sync
+        throwTo threadId E.UserInterrupt
+        r <- takeMVar mvar
         normalizeSummary r `shouldBe` [
             ""
           , "- foo FAILED [1]"
@@ -154,10 +168,17 @@ spec = do
           ]
 
       it "throws UserInterrupt" $ do
-        silence . H.hspec $ do
-          H.it "foo" $ do
-            E.throwIO E.UserInterrupt :: IO ()
-        `shouldThrow` (== E.UserInterrupt)
+        mvar <- newEmptyMVar
+        sync <- newEmptyMVar
+        threadId <- forkIO $ do
+          silence . H.hspec $ do
+            H.it "foo" $ do
+              putMVar sync ()
+              threadDelay 1000000
+          `E.catch` putMVar mvar
+        takeMVar sync
+        throwTo threadId E.UserInterrupt
+        takeMVar mvar `shouldReturn` E.UserInterrupt
 
     context "with --help" $ do
       let printHelp = withProgName "spec" . withArgs ["--help"] . H.hspec $ pure ()
@@ -294,27 +315,14 @@ spec = do
 
     context "with --seed" $ do
       it "uses specified seed" $ do
-        r <- captureLines . ignoreExitCode . withArgs ["--seed", "2413421499272008081"] . H.hspec $ do
-            H.it "foo" $
-              property (/= (26 :: Integer))
-        r `shouldContain` [
-            "Falsifiable (after 66 tests): "
-          , "26"
-          ]
+        r <- runPropFoo ["--seed", "42"]
+        r `shouldContain` prop_foo_result_with_seed_42
 
       context "when run with --rerun" $ do
         it "takes precedence" $ do
-          let runSpec args = capture_ . ignoreExitCode . withArgs args . H.hspec $ do
-                H.it "foo" $
-                  property $ \n -> ((17 + 31 * n) `mod` 50) /= (23 :: Integer)
-          r0 <- runSpec ["--seed", "23"]
-          r0 `shouldContain` "(after 88 tests)"
-
-          r1 <- runSpec ["--seed", "42"]
-          r1 `shouldContain` "(after 48 tests)"
-
-          r2 <- runSpec ["--rerun", "--seed", "23"]
-          r2 `shouldContain` "(after 88 tests)"
+          r <- runPropFoo ["--seed", "23"]
+          _ <- runPropFoo ["--seed", "42"]
+          runPropFoo ["--rerun", "--seed", "23"] `shouldReturn` r
 
       context "when given an invalid argument" $ do
         let run = withArgs ["--seed", "foo"] . H.hspec $ do
