@@ -1,11 +1,8 @@
-{-# OPTIONS_GHC -fno-warn-orphans #-}
 module Test.Hspec.UtilSpec (main, spec) where
 
 import           Helper
-import           Data.Int (Int32)
-import           System.Random (StdGen)
+import           Control.Concurrent
 import qualified Control.Exception as E
-import           System.SetEnv
 
 import           Test.Hspec.Util
 
@@ -51,8 +48,15 @@ spec = do
       Left e <- safeTry (return undefined)
       show e `shouldBe` "Prelude.undefined"
 
-    it "re-throws AsyncException" $ do
-      safeTry (E.throwIO E.UserInterrupt :: IO Int) `shouldThrow` (== E.UserInterrupt)
+    it "does not catch asynchronous exceptions" $ do
+      mvar <- newEmptyMVar
+      sync <- newEmptyMVar
+      threadId <- forkIO $ do
+        safeTry (putMVar sync () >> threadDelay 1000000) >> return ()
+        `E.catch` putMVar mvar
+      takeMVar sync
+      throwTo threadId E.UserInterrupt
+      readMVar mvar `shouldReturn` E.UserInterrupt
 
   describe "filterPredicate" $ do
     it "tries to match a pattern against a path" $ do
@@ -91,28 +95,3 @@ spec = do
 
     it "properly handles context after a subject that consists of several components" $ do
       formatRequirement (["Data", "List", "reverse", "when applied twice"], "reverses a list") `shouldBe` "Data.List.reverse, when applied twice, reverses a list"
-
-  describe "getEnv" $ do
-    it "returns value of specified environment variable" $ do
-      setEnv "FOO" "bar"
-      getEnv "FOO" `shouldReturn` Just "bar"
-
-    it "returns Nothing if specified environment variable is not set" $ do
-      unsetEnv "FOO"
-      getEnv "FOO" `shouldReturn` Nothing
-
-  describe "stdGenToInteger" $ do
-    it "is inverse to stdGenFromInteger" $ property $
-      \(NonNegative i) -> (stdGenToInteger . stdGenFromInteger) i `shouldBe` i
-
-  describe "stdGenFromInteger" $ do
-    it "is inverse to stdGenToInteger" $ property $
-      \stdGen -> (stdGenFromInteger . stdGenToInteger) stdGen `shouldBe` stdGen
-
-instance Eq StdGen where
-  a == b = show a == show b
-
-instance Arbitrary StdGen where
-  arbitrary = do
-    (Positive a, Positive b) <- arbitrary
-    return $ read (show (a :: Int32) ++ " " ++ show (b :: Int32))
